{-# LANGUAGE LambdaCase #-}
{-# LANGUAGE OverloadedStrings #-}
{-# LANGUAGE TemplateHaskell #-}

module Shift.CLI where

import Control.Lens (makeLenses)
import Options.Applicative
  ( Parser,
    ReadM,
    auto,
    command,
    eitherReader,
    help,
    info,
    long,
    metavar,
    option,
    optional,
    progDesc,
    short,
    strOption,
    subparser,
    switch,
  )

data ShiftOptions = ShiftOptions
  { _soCommand :: ShiftCommand,
    _soHostingType :: HostingType,
    _soIncludeHead :: Bool,
    _soFromRef :: Maybe String,
    _soToRef :: Maybe String,
    _soGitHubOwner :: Maybe String,
    _soGitHubRepository :: Maybe String,
    _soGitHubToken :: Maybe String
  }
  deriving (Show, Eq)

data ShiftCommand
  = GenerateCommand {_gcOutputPath :: Maybe FilePath}
  | ServeCommand {_scPort :: Maybe Int}
  deriving (Show, Eq)

data HostingType = GitHubType | GitType deriving (Show, Eq, Enum)

shiftOptions :: Parser ShiftOptions
shiftOptions =
  ShiftOptions
    <$> shiftCommand
    <*> option
      hostingType
      ( long "hosting-type"
          <> short 't'
          <> metavar "TYPE"
          <> help "Which kind of service the repository is hosted on"
      )
    <*> switch
      ( long "include-head"
          <> help "Whether to include the current HEAD as a ref in the change report"
      )
    <*> optional
      ( strOption
          ( long "from-ref"
              <> metavar "REF"
              <> help "Start ref to use for generating a change report"
          )
      )
    <*> optional
      ( strOption
          ( long "to-ref"
              <> metavar "REF"
              <> help "End ref to use for generating a change report"
          )
      )
    <*> optional
      ( strOption
          ( long "github-owner"
              <> metavar "USERNAME"
              <> help "Username who owns the repository"
          )
      )
    <*> optional
      ( strOption
          ( long "github-repository"
              <> metavar "REPOSITORY"
              <> help "Name of the repository"
          )
      )
    <*> optional
      ( strOption
          ( long "github-token"
              <> metavar "TOKEN"
              <> help "GitHub access token"
          )
      )

shiftCommand :: Parser ShiftCommand
shiftCommand =
  subparser $
    command
      "generate"
      ( info
          ( GenerateCommand
              <$> optional
                ( strOption
                    ( long "output-path" <> short 'o'
                        <> metavar "PATH"
                        <> help "File to write the report to."
                    )
                )
          )
          (progDesc "Generate changelog")
      )
      <> command
        "serve"
        ( info
            ( ServeCommand
                <$> optional
                  (option auto (long "port" <> help "Web server listener port"))
            )
            (progDesc "Start UI server")
        )

hostingType :: ReadM HostingType
hostingType = eitherReader $ \case
  "github" -> Right GitHubType
  "git" -> Right GitType
  x -> Left $ "`" ++ x ++ "` is not a supported hosting type"

makeLenses ''ShiftOptions
